TYPES = {

    # REGION
    "обл", "обл.", "область",
    # CITY
    "м", "м.", "місто", "г", "г.", "город",
    # DISTRICT
    "р-н", "р-н.", "рн", "рн.", "р-он", "район",
    # MICRODISTRICT
    "мікр", "мікр.", "мн", "мр", "мкрн", "мкр", "мікрорайон", "микр", "микр.", "микрорайон", "м-н",
    # VILLAGE
    "пос", "пос.", "смт", "смт.", "с.м.т", "пгт", "п г т", "пгт.", "село", "селище", "поселок", "с-ще",
    # STREET    
    "вул", "вул.", "вулиця", "ул", "ул.", "улица", "влу.", "в.", "вулю", "улиця",
    "пров", "пров.", "провулок", "пер", "пер.", "переулок", "прос", "провул", "прв.", "перевуло", "про",
    "бул", "бул.", "б-р", "бр", "бр.", "бур", "бур.", "бульвар", "бульв.", "бул-р.",
    "просп", "просп.", "прт", "прт.", "прокт", "прокт.", "пр", "пр.", "п-т", "п-т.", "п-рт.", "проспект", "п-т", "пр-кт", "пр-к", "прпосп"
    "ж\м" , "масив", "массив", "житловий масив", "жилой массив", "ж.м.",
    "ш.", "шосе", "шоссе",
    "алея", "аллея",
    "майд", "майд.", "майдан",
    "розвилка", "развилка",
    "узвіз", "спуск",
    "проїзд", "проезд",
    "дорога",
    "наб", "наб.", "набер.", "набережна", "набережная",
    "парк",
    "сквер",
    "тупик",
    "прохід", "проход",
    "ст", "ст.", "станція", "станция",
    "остр", "остр.", "острів", "остров",
    "шлях", "путь",
    "гай", "роща", 
    "пл", "пл.", "площа", "площадь",
    "в'їзд", "въезд",
    "лінія", "линия",
    "траса", "трасса",
    "урочище",
    "шахта",
    "хутір", "хутор",
    "роз'їзд", "разъезд",
    "квартал",
    # HOUSING
    "корп.", "корп", "корпус",
    # HOSTEL
    "гурт", "гурт.", "гуртожиток", "общ", "общ.", "общежитие",
    # HOUSE
    "буд.", "будинок", "дом", "д.", "б.",
    # APARTMENT
    "кв.", "квартира",
    # ROOM
    "прим.",
    # SECTION
    "секція",
    # ENTRANCE
    "підʼїзд", "подъезд",
    # FLOOR
    "поверх", "этаж"
    
}